@@ -120,7 +120,8 @@ def serialize_class(Cls):
     Serialize an xtUML metamodel class.
     '''
     metaclass = xtuml.get_metaclass(Cls)
-    attributes = ['%s %s' % (name, ty.upper()) for name, ty in metaclass.attributes]
+    attributes = ['%s %s' % (name, ty.upper())
+                  for name, ty in metaclass.attributes]
     
     s = 'CREATE TABLE %s (\n    ' % metaclass.kind
     s += ',\n    '.join(attributes)
@@ -130,8 +131,8 @@ def serialize_class(Cls):
 
 def serialize_unique_identifiers(metamodel):
     s = ''
-    
-    for metaclass in metamodel.metaclasses.values():
+    for kind in sorted(metamodel.metaclasses.keys()):
+        metaclass = metamodel.metaclasses[kind]
         for index_name, attribute_names in metaclass.indices.items():
             attribute_names = ', '.join(attribute_names)
             s += 'CREATE UNIQUE INDEX %s ON %s (%s);\n' % (index_name,
@@ -139,18 +140,29 @@ def serialize_unique_identifiers(metamodel):
                                                           attribute_names)
     return s
 
+
+def serialize_classes(metamodel):
+    '''
+    Serialize all class definitions in a *metamodel*.
+    '''
+    return ''.join([serialize_class(metamodel.metaclasses[kind].clazz)
+                    for kind in sorted(metamodel.metaclasses.keys())])
+
+
+def serialize_associations(metamodel):
+    '''
+    Serialize all association definitions in a *metamodel*.
+    '''
+    orderby = lambda x: (x.rel_id, x.target_link.from_metaclass.kind)
+    return ''.join([serialize_association(ass)
+                    for ass in sorted(metamodel.associations, key=orderby)])
+
+
 def serialize_schema(metamodel):
     '''
     Serialize all class and association definitions in a *metamodel*.
     '''
-    s = ''
-    for kind in sorted(metamodel.metaclasses.keys()):
-        s += serialize_class(metamodel.metaclasses[kind].clazz)
-    
-    for ass in sorted(metamodel.associations, key=lambda x: x.rel_id):
-        s += serialize_association(ass)
-    
-    return s
+    return serialize_classes(metamodel) + serialize_associations(metamodel)
 
 
 def serialize_database(metamodel):
